@@ -1,4 +1,4 @@
-/* $Id: color.h,v 1.2 2001/06/03 22:16:28 skids Exp $
+/* $Id: color.h,v 1.4 2002/09/29 19:27:41 skids Exp $
 ******************************************************************************
 
    Generic color library defines
@@ -29,13 +29,16 @@
 #ifndef _LIBGGI_DEFAULT_COLOR_H
 #define _LIBGGI_DEFAULT_COLOR_H
 
+#include "config.h"
 #include <ggi/internal/ggi-dl.h>
 
 ggifunc_mapcolor	GGI_color_PAL_mapcolor;
+ggifunc_mapcolor	GGI_color_TRUE16_mapcolor;
 ggifunc_mapcolor	GGI_color_TRUE_mapcolor;
 ggifunc_mapcolor	GGI_color_GREY_mapcolor;
 
 ggifunc_unmappixel	GGI_color_PAL_unmappixel;
+ggifunc_unmappixel	GGI_color_TRUE16_unmappixel_4to7;
 ggifunc_unmappixel	GGI_color_TRUE_unmappixel_gte8;
 ggifunc_unmappixel	GGI_color_TRUE_unmappixel_gte4;
 ggifunc_unmappixel	GGI_color_TRUE_unmappixel_gte2;
