@@ -1,4 +1,4 @@
-/* $Id: clip.c,v 1.1.1.1 2001/05/12 23:01:33 cegger Exp $
+/* $Id: clip.c,v 1.4.2.1 2003/02/05 13:51:50 cegger Exp $
 ******************************************************************************
 
    Graphics library for GGI.
@@ -25,6 +25,7 @@
 ******************************************************************************
 */
 
+#include "config.h"
 #include <ggi/internal/ggi-dl.h>
 
 /*
@@ -113,8 +114,8 @@ static int _ggi_clip2d(ggi_visual *vis,int *_x0, int *_y0, int *_x1, int *_y1,
 	int xmajor;
 	int slope;
 
-	first = 0;
-	last = 0;
+	*clip_first = first = 0;
+	*clip_last = last = 0;
 	outcode(first,*_x0,*_y0);
 	outcode(last,*_x1,*_y1);
 
@@ -208,7 +209,7 @@ static int _ggi_clip2d(ggi_visual *vis,int *_x0, int *_y0, int *_x1, int *_y1,
 			outcode(last,x1,y1);
 			*clip_last = 1;
 		}
-    
+
 		if ((first & last) != 0) {
 			return 0; /* Trivially rejected! */
 		}
