@@ -1,4 +1,4 @@
-/* $Id: crossblit.c,v 1.7.2.1 2003/04/12 22:36:28 cegger Exp $
+/* $Id: crossblit.c,v 1.7.2.2 2004/03/03 00:21:46 cegger Exp $
 ******************************************************************************
 
    16-bpp linear direct-access framebuffer renderer for LibGGI:
@@ -90,7 +90,7 @@ crossblit_same(ggi_visual *src, int sx, int sy, int w, int h,
 	w *= 2;		/* Use width in bytes. */
 
 	for (; h != 0; h--) {
-		memcpy(dstp, srcp, w);
+		memcpy(dstp, srcp, (size_t)(w));
 		srcp += srcstride;
 		dstp += dststride;
 	}
@@ -112,7 +112,7 @@ cb4to16(ggi_visual *src, int sx, int sy, int w, int h,
 	GGIDPRINT_DRAW("linear-16: cb4to16.\n");
 
 	do {
-		int i;
+		unsigned int i;
 		for (i = 0; i < 16; i++) {
 			ggi_color col;
 
@@ -209,7 +209,7 @@ cb8to16(ggi_visual *src, int sx, int sy, int w, int h,
 
 	/* Create a conversion table. */
 	do {
-		int i;
+		unsigned int i;
 		for (i = 0; i < 256; i++) {
 			ggi_color col;
 
@@ -974,6 +974,7 @@ static inline void cb16to16_64bitc(ggi_visual *src, int sx, int sy, int w,
 				}
 			}
 
+			if (stopcol <= dstp) break;
 			tmp = 0;
 			cache = *srcp;
 			i = 0;
@@ -1092,16 +1093,16 @@ static inline void cb32to16_64bitc(ggi_visual *src, int sx, int sy, int w,
 				done64:
 #ifdef GGI_LITTLE_ENDIAN
 					tmpa64 |= tmpa64 >> 16;
-					tmpa64 &= 0x00000000ffffffff;
+					tmpa64 &= 0x00000000ffffffffLL;
 					tmpa64 |= tmpb64 << 32;
-					tmpb64 &= 0x0000ffff00000000;
+					tmpb64 &= 0x0000ffff00000000LL;
 					tmpa64 |= tmpb64 << 16;
 					*((uint64 *)dstp) = tmpa64;
 #else
 					tmpb64 |= tmpa64 >> 16;
-					tmpb64 &= 0x00000000ffffffff;
+					tmpb64 &= 0x00000000ffffffffLL;
 					tmpb64 |= tmpa64 << 32;
-					tmpa64 &= 0x0000ffff00000000;
+					tmpa64 &= 0x0000ffff00000000LL;
 					tmpb64 |= tmpa64 << 16;
 					*((uint64 *)dstp) = tmpb64;
 #endif
@@ -1124,6 +1125,7 @@ static inline void cb32to16_64bitc(ggi_visual *src, int sx, int sy, int w,
 				}
 			}
 
+			if (stopcol <= dstp) break;
 			tmp = 0;
 			cache = *srcp;
 			i = 0;
@@ -1276,7 +1278,6 @@ static inline void cb16to16_mmx(ggi_visual *src, int sx, int sy, int w, int h,
 			if ((stopcol > dstp + 11) && 
 			    !((unsigned int)dstp % 8)) {
 				while (stopcol > dstp + 11) {
-				  void *dummy;
 				  __asm__ __volatile__(
 				   "movq  (%1), %%mm0\n\t"
                                    "pxor  %%mm5, %%mm5\n\t"
@@ -1287,28 +1288,28 @@ static inline void cb16to16_mmx(ggi_visual *src, int sx, int sy, int w, int h,
                                    "add   $24, %1\n\t"
 
                                    ".Lleft%=:"
-                                   "cmp   $0, 8(%2)\n\t"
+                                   "cmp   $0, 8(%4)\n\t"
                                    "je    .Lmiddle%=\n\t"
 
                                    "movq  %%mm0, %%mm3\n\t"
                                    "movq  %%mm1, %%mm4\n\t"
-                                   "psllw (%2), %%mm3\n\t"
-                                   "psllw (%2), %%mm4\n\t"
-                                   "pand  8(%2), %%mm3\n\t"
-                                   "pand  8(%2), %%mm4\n\t"
+                                   "psllw (%4), %%mm3\n\t"
+                                   "psllw (%4), %%mm4\n\t"
+                                   "pand  8(%4), %%mm3\n\t"
+                                   "pand  8(%4), %%mm4\n\t"
                                    "por   %%mm3, %%mm5\n\t"
                                    "movq  %%mm2, %%mm3\n\t"
                                    "por   %%mm4, %%mm6\n\t"
-                                   "psllw (%2), %%mm3\n\t"
-                                   "pand  8(%2), %%mm3\n\t"
-                                   "cmp   $0, (%2)\n\t"
+                                   "psllw (%4), %%mm3\n\t"
+                                   "pand  8(%4), %%mm3\n\t"
+                                   "cmp   $0, (%4)\n\t"
                                    "por   %%mm3, %%mm7\n\t"
                                    "je    .Lright%=\n\t"
-                                   "add   $16, %2\n\t"
+                                   "add   $16, %4\n\t"
                                    "jmp   .Lleft%=\n\t"
 
                                    ".Lmiddle%=:\n\t"
-                                   "cmp   $0, (%2)\n\t"
+                                   "cmp   $0, (%4)\n\t"
                                    "je    .Lright%=\n\t"
 
                                    ".Ldone%=:\n\t"
@@ -1319,33 +1320,34 @@ static inline void cb16to16_mmx(ggi_visual *src, int sx, int sy, int w, int h,
                                    "jmp   .Lout%=\n\t"
 
                                    ".Lright%=:\n\t"
-                                   "add   $16, %2\n\t"
-                                   "cmp   $0, 8(%2)\n\t"
+                                   "add   $16, %4\n\t"
+                                   "cmp   $0, 8(%4)\n\t"
                                    "je    .Ldone%=\n\t"
 
                                    "movq  %%mm0, %%mm3\n\t"
                                    "movq  %%mm1, %%mm4\n\t"
-                                   "psrlw (%2), %%mm3\n\t"
-                                   "psrlw (%2), %%mm4\n\t"
-                                   "pand  8(%2), %%mm3\n\t"
-                                   "pand  8(%2), %%mm4\n\t"
+                                   "psrlw (%4), %%mm3\n\t"
+                                   "psrlw (%4), %%mm4\n\t"
+                                   "pand  8(%4), %%mm3\n\t"
+                                   "pand  8(%4), %%mm4\n\t"
                                    "por   %%mm3, %%mm5\n\t"
                                    "movq  %%mm2, %%mm3\n\t"
                                    "por   %%mm4, %%mm6\n\t"
-                                   "psrlw (%2), %%mm3\n\t"
-                                   "pand  8(%2), %%mm3\n\t"
+                                   "psrlw (%4), %%mm3\n\t"
+                                   "pand  8(%4), %%mm3\n\t"
                                    "por   %%mm3, %%mm7\n\t"
 
                                    "jmp   .Lright%=\n\t"
 
                                    ".Lout%=:\n\t"
                                    "emms\n\t"
-				   : "=qom" (dstp), "=qom" (srcp), "=q" (dummy)
-				   : "qom" (dstp), "qom" (srcp), "q" (tab)
+				   : "=q" (dstp), "=q" (srcp)
+				   : "0" (dstp), "1" (srcp), "q" (tab)
 				   : "cc", "memory");
 				}
 			}
 
+			if (stopcol <= dstp) break;
 			tmp = 0;
 			cache = *srcp;
 			i = 0;
@@ -1435,7 +1437,6 @@ static inline void cb32to16_mmx(ggi_visual *src, int sx, int sy, int w, int h,
 			if ((stopcol > dstp + 7) && 
 			    !((unsigned int)dstp % 8)) {
 				while (stopcol > dstp + 7) {
-				  void *dummy;
 				  __asm__ __volatile__(
 
 				   "movq  (%1), %%mm0\n\t"
@@ -1447,34 +1448,34 @@ static inline void cb32to16_mmx(ggi_visual *src, int sx, int sy, int w, int h,
                                    "pxor  %%mm6, %%mm6\n\t"
 
                                    ".Lleft%=:"
-                                   "cmp   $0, 8(%2)\n\t"
+                                   "cmp   $0, 8(%4)\n\t"
                                    "je    .Lmiddle%=\n\t"
 
                                    "movq  %%mm1, %%mm4\n\t"
                                    "movq  %%mm0, %%mm5\n\t"
-                                   "pslld (%2), %%mm4\n\t"
-                                   "pslld (%2), %%mm5\n\t"
+                                   "pslld (%4), %%mm4\n\t"
+                                   "pslld (%4), %%mm5\n\t"
                                    "psrad $16, %%mm4\n\t"
                                    "psrad $16, %%mm5\n\t"
                                    "packssdw %%mm4, %%mm5\n\t"
                                    "movq  %%mm3, %%mm4\n\t"
-                                   "pand  8(%2), %%mm5\n\t"
+                                   "pand  8(%4), %%mm5\n\t"
                                    "por   %%mm5, %%mm6\n\t"
                                    "movq  %%mm2, %%mm5\n\t"
-                                   "pslld (%2), %%mm4\n\t"
-                                   "pslld (%2), %%mm5\n\t"
+                                   "pslld (%4), %%mm4\n\t"
+                                   "pslld (%4), %%mm5\n\t"
                                    "psrad $16, %%mm4\n\t"
                                    "psrad $16, %%mm5\n\t"
                                    "packssdw %%mm4, %%mm5\n\t"
-                                   "pand  8(%2), %%mm5\n\t"
-                                   "cmp   $0, (%2)\n\t"
+                                   "pand  8(%4), %%mm5\n\t"
+                                   "cmp   $0, (%4)\n\t"
                                    "por   %%mm5, %%mm7\n\t"
                                    "je    .Lright%=\n\t"
-                                   "add   $16, %2\n\t"
+                                   "add   $16, %4\n\t"
                                    "jmp   .Lleft%=\n\t"
 
                                    ".Lmiddle%=:\n\t"
-                                   "cmp   $0, (%2)\n\t"
+                                   "cmp   $0, (%4)\n\t"
                                    "je    .Lright%=\n\t"
 
                                    ".Ldone%=:\n\t"
@@ -1484,35 +1485,35 @@ static inline void cb32to16_mmx(ggi_visual *src, int sx, int sy, int w, int h,
                                    "jmp   .Lout%=\n\t"
 
                                    ".Lright%=:\n\t"
-                                   "add   $16, %2\n\t"
-                                   "cmp   $0, 8(%2)\n\t"
+                                   "add   $16, %4\n\t"
+                                   "cmp   $0, 8(%4)\n\t"
                                    "je    .Ldone%=\n\t"
 
                                    "movq  %%mm1, %%mm4\n\t"
                                    "movq  %%mm0, %%mm5\n\t"
-                                   "psrld (%2), %%mm4\n\t"
-                                   "psrld (%2), %%mm5\n\t"
+                                   "psrld (%4), %%mm4\n\t"
+                                   "psrld (%4), %%mm5\n\t"
                                    "packssdw %%mm4, %%mm5\n\t"
                                    "movq  %%mm3, %%mm4\n\t"
-                                   "pand  8(%2), %%mm5\n\t"
+                                   "pand  8(%4), %%mm5\n\t"
                                    "por   %%mm5, %%mm6\n\t"
                                    "movq  %%mm2, %%mm5\n\t"
-                                   "psrld (%2), %%mm4\n\t"
-                                   "psrld (%2), %%mm5\n\t"
+                                   "psrld (%4), %%mm4\n\t"
+                                   "psrld (%4), %%mm5\n\t"
                                    "packssdw %%mm4, %%mm5\n\t"
-                                   "pand  8(%2), %%mm5\n\t"
+                                   "pand  8(%4), %%mm5\n\t"
                                    "por   %%mm5, %%mm7\n\t"
                                    "jmp   .Lright%=\n\t"
 
                                     ".Lout%=:\n\t"
                                    "emms\n\t"
-				   : "=qom" (dstp), "=qom" (srcp), "=q" (dummy)
-				   : "qom" (dstp), "qom" (srcp), "q" (tab)
+				   : "=q" (dstp), "=q" (srcp)
+				   : "0" (dstp), "1" (srcp), "q" (tab)
 				   : "cc", "memory");
 				}
 			}
 
-			
+			if (stopcol <= dstp) break;
 			tmp = 0;
 			cache = *srcp;
 			i = 0;
