@@ -1,4 +1,4 @@
-/* $Id: crossblit.c,v 1.1.1.1 2001/05/12 23:01:46 cegger Exp $
+/* $Id: crossblit.c,v 1.4 2002/11/16 19:06:30 skids Exp $
 ******************************************************************************
 
    Graphics library for GGI.
@@ -29,6 +29,7 @@
 */
 
 #include <string.h>
+#include "config.h"
 #include <ggi/internal/ggi-dl.h>
 
 
@@ -42,6 +43,8 @@ fallback(ggi_visual *src, int sx, int sy, int w, int h,
 	uint8 *dstptr;
 	int stride;
 
+	GGIDPRINT_DRAW("linear-8: fallback to slow crossblit.\n");
+
 	LIBGGIGetPixel(src, sx, sy, &cur_src);
 	cur_src++; /* assure safe init */
 	
@@ -106,6 +109,7 @@ crossblit_8_to_8(ggi_visual *src, int sx, int sy, int w, int h,
 		 * compiler hence unroll manually.
 		 */
 		switch (w & 0x7) {
+		default:
 			for (; i > 0; i--) {
 			case 0x0: *dstp++ = conv_tab[*srcp++];
 			case 0x7: *dstp++ = conv_tab[*srcp++];
