@@ -1,4 +1,4 @@
-/* $Id: mode.c,v 1.11.2.2 2003/05/01 21:58:01 cegger Exp $
+/* $Id: mode.c,v 1.11.2.4 2004/05/01 21:42:37 cegger Exp $
 ******************************************************************************
 
    Graphics library for GGI. X target.
@@ -328,9 +328,16 @@ int GGI_X_setmode_normal(ggi_visual *vis, ggi_mode *tm)
 	/* Wait for window to become mapped */
 	XNextEvent (priv->disp, &event);
 	GGIDPRINT_MODE("X: Window Mapped\n");
-	
-	/* Parent window doesn't listen for input. */
-	XSelectInput(priv->disp, priv->parentwin, 0);
+
+	/* We let the parent window listen for the keyboard.
+	 * this allows to have the windowmanager decide about keyboard 
+	 * focus as usual.
+	 * The child window listens for the rest.
+	 * Note, that the child window also listens for the keyboard for
+	 * those cases where we don't have a parent.
+	 */
+	XSelectInput(priv->disp, priv->parentwin,
+		     KeyPressMask | KeyReleaseMask);
 
 oldparent:
 	GGIDPRINT_MODE("X: running in parent window 0x%x\n", priv->parentwin);
@@ -356,8 +363,8 @@ oldparent:
 	attrib.colormap = priv->cmap;
 	attrib.border_pixel = BlackPixel(priv->disp, vi->screen);
 	priv->win = XCreateWindow(priv->disp, priv->parentwin,
-				  0, 0, tm->virt.x, 
-				  tm->virt.y * (tm->frames + 1), 0,
+				  0, 0, (unsigned)tm->virt.x, 
+				  (unsigned)tm->virt.y * tm->frames, 0,
 				  vi->depth, InputOutput,
 				  vi->visual, CWColormap | CWBorderPixel, 
 				  &attrib);
@@ -394,7 +401,7 @@ oldparent:
 		XSetFont(priv->disp, priv->tempgc, priv->textfont->fid);
 	_ggi_x_set_xclip(NULL, priv->disp, priv->tempgc, 0, 0, 
 			 LIBGGI_VIRTX(vis), 
-			 LIBGGI_VIRTY(vis) * (vis->mode->frames + 1));
+			 LIBGGI_VIRTY(vis) * vis->mode->frames);
 	GGIDPRINT_MODE("X GCs allocated.\n");
 
 	/* Create a cursor (frees old cursor) */
@@ -427,6 +434,7 @@ oldparent:
 		data->win = priv->win;
 		if (data->win == None) data->win = priv->parentwin;
 		data->ptralwaysrel = 0;
+		data->parentwin = priv->parentwin;
 
 		giiEventSend(priv->inp, &ev);
 	}
@@ -549,7 +557,7 @@ int GGI_X_setmode_fixed(ggi_visual *vis, ggi_mode *tm)
 	XSetGraphicsExposures(priv->disp, priv->tempgc, True);
 	_ggi_x_set_xclip(NULL, priv->disp, priv->tempgc, 0, 0, 
 			 LIBGGI_VIRTX(vis), 
-			 LIBGGI_VIRTY(vis) * (vis->mode->frames + 1));
+			 LIBGGI_VIRTY(vis) * vis->mode->frames);
 	GGIDPRINT_MODE("X GCs allocated.\n");
 
 	/* Create a cursor (destroys old one) */
