@@ -1,4 +1,4 @@
-/* $Id: visual.c,v 1.13.2.3 2003/03/19 20:36:33 cegger Exp $
+/* $Id: visual.c,v 1.13.2.4 2004/01/28 08:35:44 cegger Exp $
 ******************************************************************************
 
    LibGGI Display-X target: initialization
@@ -71,7 +71,8 @@ static const gg_option optlist[] =
 #define NUM_OPTS	(sizeof(optlist)/sizeof(gg_option))
 
 
-int GGI_X_getapi(ggi_visual *vis,int num, char *apiname ,char *arguments)
+static int GGI_X_getapi(ggi_visual *vis,int num,
+			char *apiname ,char *arguments)
 {
 	*arguments = '\0';
 	switch (num) {
@@ -323,9 +324,13 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 		}
 	} else {
 		XWindowAttributes wa;
-		vi_template.screen = (options[OPT_SCREEN].result[0] != 'n')  ? 
-		  strtoul(options[OPT_SCREEN].result, NULL, 0) : 
-		  DefaultScreen(priv->disp);
+		if (options[OPT_SCREEN].result[0] != 'n') {
+			vi_template.screen =
+				strtoul(options[OPT_SCREEN].result, NULL, 0);
+		} else {
+			vi_template.screen = DefaultScreen(priv->disp);
+		}
+
 		priv->win = priv->parentwin = 
 			RootWindow(priv->disp, vi_template.screen);
 		XGetWindowAttributes(priv->disp, priv->win, &wa);
@@ -345,7 +350,7 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
 		err = GGI_ENOFUNC;
 		goto out;
 	}
-	priv->vilist = calloc(priv->nvisuals, sizeof(ggi_x_vi));
+	priv->vilist = calloc((size_t)priv->nvisuals, sizeof(ggi_x_vi));
 	if (priv->vilist == NULL) goto out;
 
 	priv->buflist = XListPixmapFormats(disp, &(priv->nbufs));
@@ -471,20 +476,21 @@ static int GGIopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
         }
 
 	if (tolower((int)options[OPT_NOINPUT].result[0]) == 'n') {
-		gii_inputxwin_arg args;
+		gii_inputxwin_arg _args;
 		gii_input *inp;
                 
-		args.disp = priv->disp;
-		args.ptralwaysrel = 0;
-		args.wait = 1;
-                args.exposefunc = (gii_inputxwin_exposefunc*)GGI_X_expose;
-                args.exposearg = vis;
-      /* args.resizefunc = (gii_inputxwin_resizefunc*)GGI_X_resize;*/
-		args.resizefunc = NULL;
-                args.resizearg = vis;
-		args.gglock = lock;
+		_args.disp = priv->disp;
+		_args.ptralwaysrel = 0;
+		_args.wait = 1;
+                _args.exposefunc = (gii_inputxwin_exposefunc*)GGI_X_expose;
+		if (priv->createfb == NULL) _args.exposefunc = NULL;
+		_args.exposearg = vis;
+      /* _args.resizefunc = (gii_inputxwin_resizefunc*)GGI_X_resize;*/
+		_args.resizefunc = NULL;
+                _args.resizearg = vis;
+		_args.gglock = lock;
                 
-		if ((inp = giiOpen("xwin", &args, NULL)) == NULL) {
+		if ((inp = giiOpen("xwin", &_args, NULL)) == NULL) {
 			GGIDPRINT_MISC("Unable to open xwin inputlib\n");
 			GGIclose(vis, dlh);
 			return GGI_ENODEVICE;
@@ -517,6 +523,8 @@ static int GGIexit(ggi_visual *vis, struct ggi_dlhandle *dlh)
 	return 0;
 }
 
+int GGIdl_X(int func, void **funcptr);
+
 int GGIdl_X(int func, void **funcptr)
 {
 	switch (func) {
