@@ -1,9 +1,10 @@
-/* $Id: libtele.c,v 1.1.1.1 2001/05/12 23:02:31 cegger Exp $
+/* $Id: libtele.c,v 1.5.2.2 2003/10/12 10:10:34 cegger Exp $
 ******************************************************************************
 
    libtele.c
 
    Copyright (C) 1998 Andrew Apted	[andrew@ggi-project.org]
+                 2002 Tobias Hunger     [tobias@fresco.org]
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -25,6 +26,7 @@
 ******************************************************************************
 */
 
+#include "config.h"
 #include <ggi/internal/plat.h>
 
 #include <stdio.h>
@@ -505,8 +507,13 @@ static int tclient_open_inet(TeleClient *c, const char *addr)
 
 
 	/* Obtain numerical address from string */
-	strncpy(name, addr, n);
-	name[n] = '\0';
+	if ((size_t)(n) < sizeof(name)) {
+		strncpy(name, addr, (size_t)(n));
+		name[n] = '\0';
+	} else {
+		strncpy(name, addr, sizeof(name));
+		name[sizeof(name) - 1] = '\0';
+	}
 
 	h = gethostbyname(name);
 
@@ -677,7 +684,8 @@ int tserver_init(TeleServer *s, int display)
 	s->endianness = MY_ENDIAN;
 
 	if (s->inet) {
-		fprintf(stderr, "tserver: Creating inet socket...\n");
+		fprintf(stderr, "tserver: Creating inet socket [%d]\n",
+				TELE_PORT_BASE + display);
 
 		me_in.sin_family      = AF_INET;
 		me_in.sin_port        = htons(TELE_PORT_BASE + display);
@@ -690,7 +698,8 @@ int tserver_init(TeleServer *s, int display)
 	
 	} else {
 #ifdef HAVE_UNIX_DOMAIN_SOCKET
-		fprintf(stderr, "tserver: Creating unix socket...\n");
+		fprintf(stderr, "tserver: Creating unix socket... [%d]\n",
+				TELE_PORT_BASE + display);
 
 		me_un.sun_family = AF_UNIX;
 
@@ -734,7 +743,13 @@ int tserver_exit(TeleServer *s)
 	if (! s->inet) {
 		char filename[200];
 
-		sprintf(filename, "%s%d", TELE_FIFO_BASE, s->display);
+#ifdef HAVE_SNPRINTF
+		snprintf(filename, 200, "%s%d",
+			TELE_FIFO_BASE, s->display);
+#else
+		sprintf(filename, "%s%d",
+			TELE_FIFO_BASE, s->display);
+#endif
 
 		unlink(filename);
 	}
