@@ -1,4 +1,4 @@
-/* $Id: structs.c,v 1.1.1.1 2001/05/12 23:03:32 cegger Exp $
+/* $Id: structs.c,v 1.1.1.1.2.1 2003/02/14 17:35:32 cegger Exp $
 ******************************************************************************
 
    This is a GGI test application.
@@ -122,6 +122,11 @@ void
 print_pixfmt(const ggi_pixelformat *pixfmt)
 {
 	int i;
+	if (pixfmt == NULL) {
+		fprintf(stderr, "  pixfmt is a NULL pointer!\n");
+		fflush(stderr);
+		return;
+	}
 
 	printf("  depth = %2d, size = %2d, flags = 0x%08x\n"
 	       "  stdformat = 0x%08x\n"
